@@ -14,15 +14,18 @@ Copyright of the modifications 1998.
 Modified by Cornelius Krasel <krasel@wpxx02.toxi.uni-wuerzburg.de>
 and Kazushi (Jam) Marukawa <jam@pobox.com>.
 Copyright of the modifications 1998, 1999.
-Modified by Matthias Andree <matthias.andree@web.de>
-Copyright of the modifications 2000 - 2002.
 Modified by Ralf Wildenhues <ralf.wildenhues@gmx.de>
-Copyright of the modification 2002.
+Copyright of the modifications 2002.
+Modified by Richard van der Hoff <richard@rvanderhoff.org.uk>
+Copyright of the modifications 2002.
+Modified by Matthias Andree <matthias.andree@web.de>
+Copyright of the modifications 2000 - 2003.
 
 See file COPYING for restrictions on the use of this software.
 */
 
 #include "leafnode.h"
+#include "mysigact.h"
 
 #include <sys/types.h>
 #include <signal.h>
@@ -31,11 +34,11 @@ See file COPYING for restrictions on the use of this software.
 #include <setjmp.h>
 #include <stdio.h>
 
-jmp_buf timeout;
+sigjmp_buf timeout;
 
 RETSIGTYPE timer(int sig)
 {
-    longjmp(timeout, 1);
+    siglongjmp(timeout, 1);
     exit(sig);			/* not reached */
 }
 
@@ -47,12 +50,13 @@ mgetaline(FILE * f)
 {
     char *l;
 
-    if (setjmp(timeout)) {
+    if (sigsetjmp(timeout,1)) {
 	return NULL;
     }
-    (void)signal(SIGALRM, timer);
+    mysigact(SIGALRM, SA_RESETHAND, timer, 0);
     (void)alarm(timeout_client);
     l = getaline(f);
     (void)alarm(0U);
+    mysigact(SIGALRM, 0, SIG_DFL, 0);
     return l;
 }
