@@ -18,7 +18,7 @@ Modified by Matthias Andree <matthias.andree@web.de>
 Copyright of the modifications 2000 - 2002.
 Modified by Ralf Wildenhues <ralf.wildenhues@gmx.de>
 Copyright of the modifications 2002.
-Modified by Jonathan Larmour <jlarmour@redhat.com>
+Modified by Jonathan Larmour <jifl@jifvik.org>
 Copyright of the modifications 2002.
 
 See file COPYING for restrictions on the use of this software.
@@ -30,6 +30,7 @@ See file COPYING for restrictions on the use of this software.
 #include "validatefqdn.h"
 #include "strlcpy.h"
 #include "ln_log.h"
+#include "nntpd.h"
 
 #ifdef SOCKS
 #include <socks.h>
@@ -42,7 +43,7 @@ See file COPYING for restrictions on the use of this software.
 #include <arpa/inet.h>
 #endif
 #include <ctype.h>
-#include <dirent.h>
+#include "system.h"
 #include <errno.h>
 #include <fcntl.h>
 #include <limits.h>
@@ -53,8 +54,6 @@ See file COPYING for restrictions on the use of this software.
 #include <string.h>
 #include <sys/socket.h>
 #include <syslog.h>
-#include <time.h>
-#include <sys/time.h>
 #include <unistd.h>
 #include <utime.h>
 
@@ -79,7 +78,7 @@ void list(struct newsgroup *ng, int what, const char *pattern);
 void rereadactive(void);
 
 void parser(void);
-void error(const char *msg);
+void myerror(const char *msg);
 void doarticle(const char *arg, int what);
 int dogroup(const char *);
 void dohelp(void);
@@ -146,7 +145,7 @@ rereadactive(void)
 
 
 void
-error(const char *msg)
+myerror(const char *msg)
 {
     printf("%s: %s\r\n", msg, strerror(errno));
 }
@@ -212,6 +211,8 @@ parser(void)
 	    domove(1);
 	} else if (!strcasecmp(cmd, "list")) {
 	    dolist(arg);
+	} else if (!strcasecmp(cmd, "date")) {
+	    dodate();
 	} else if (!strcasecmp(cmd, "mode")) {
 	    if (debugmode)
 		syslog(LOG_DEBUG, ">200 Leafnode %s, pleased to meet you!",
@@ -339,8 +340,8 @@ fopenpseudoart(const char *arg, const unsigned long article_num)
     char *c;
     struct newsgroup *g;
 
-    if (group && article_num && article_num == group->first &&
-	    group->first >= group->last) {
+    if (group && (article_num && ((article_num == group->first &&
+	    group->first >= group->last) || is_pseudogroup(group)))) {
 	f = buildpseudoart(group->name);
     } else if (!article_num) {
 	if (!strncmp(arg, "<leafnode:placeholder:", 22)) {
@@ -521,6 +522,7 @@ doarticle(const char *arg, int what)
 			markgroup = group->name;
 			break;
 		    }
+		    p = q + strcspn(q, " \t");
 		}
 	    }
 	    /* if we don't have a localartno, then we need to mark this
@@ -550,7 +552,13 @@ doarticle(const char *arg, int what)
     }
 
     if (!localmsgid) {
-	printf("503 internal error %s:%lu", __FILE__, (unsigned long)__LINE__);
+	const char *tmp = "423 Corrupt article.";
+	printf("%s\r\n", tmp);
+	syslog(LOG_WARNING, ">%s", tmp);
+	if (replyartno) {
+	    (void)xsnprintf(s, SIZE_s, "%lu", replyartno);
+	    (void)log_unlink(s, 0);
+	}
 	(void)fclose(f);
 	if (xref)
 	    free(xref);
@@ -595,7 +603,7 @@ doarticle(const char *arg, int what)
 	/* delaybody:
 	   t == NULL: body is missing, mark for download
 	   t != NULL: body is present */
-	if (delaybody && t == NULL) {
+	if (t == NULL) {
 	    if (localartno && markgroup != NULL) {
 		switch (markdownload(markgroup, localartno)) {
 		case 0:
@@ -669,7 +677,7 @@ markinterest(const char *ng)
     err = 0;
     (void)xsnprintf(s, SIZE_s, "%s/interesting.groups/%s", spooldir, ng);
     if (stat(s, &st) == 0) {
-	now = time(0);
+	now = time(NULL);
 	buf.actime = (now < st.st_atime) ? st.st_atime : now;
 	/* now < update may happen through HW failures */
 	buf.modtime = st.st_mtime;
@@ -691,7 +699,7 @@ markinterest(const char *ng)
 	}
     } else {
 	int fd = open(".", O_RDONLY);
-	(void)chdirgroup(ng, TRUE);
+	(void)chdirgroup(ng, FALSE);
 	if (fd >= 0) {
 	    (void)fchdir(fd);
 	    (void)close(fd);
@@ -717,6 +725,7 @@ dogroup(const char *arg)
 		   g->last >= g->first ? g->last - g->first + 1 : 0,
 		   g->first, g->last, g->name);
 	} else {
+	    g->first = g->last;
 	    if (debugmode)
 		syslog(LOG_DEBUG,
 		       ">211 %lu %lu %lu %s group selected (pseudo article)",
@@ -740,6 +749,7 @@ dohelp(void)
     fputs("100 Legal commands on THIS server:\r\n"
 	    " ARTICLE [<Message-ID>|<Number>]\r\n"
 	    " BODY [<Message-ID>|<Number>]\r\n"
+	    " DATE\r\n"
 	    " GROUP <Newsgroup>\r\n"
 	    " HDR <Header> <Message-ID>|<Range>\r\n"
 	    " HEAD [<Message-ID>|<Number>]\r\n"
@@ -828,7 +838,7 @@ list(struct newsgroup *g, int what, const char *pattern)
 		    printf("%s\t%s\r\n", g->name, g->desc ? g->desc : "-x-");
 		    break;
 		case 2:
-		    printf("%s %lu news@%s\r\n", g->name, g->age, fqdn);
+		    printf("%s %lu news@%s\r\n", g->name, (unsigned long)g->age, fqdn);
 		    break;
 		default:
 		    break;
@@ -1023,7 +1033,8 @@ generateMessageID(void)
     char *p;
     unsigned long n;
 
-    now = time(0);		/* might be 0, in which case fudge will almost fix it */
+    now = time(NULL);		/* might be 0, in which case fudge
+				   will almost fix it */
     if (now < OFFSET) {
 	ln_log(LNLOG_SCRIT, LNLOG_CTOP,
 		"your system clock cannot be right. abort.");
@@ -1078,7 +1089,7 @@ dopost(void)
 	(void)xsnprintf(outname, sizeof(outname), "%s/out.going/%d-%d-%d",
 		spooldir, (int)getpid(), (int)time(NULL), ++postingno);
 
-	o = open(outname, O_WRONLY | O_EXCL | O_CREAT, 0644);
+	o = open(outname, O_WRONLY | O_EXCL | O_CREAT, 0244);
 	if (o < 0 && errno != EEXIST) {
 	    char *errmsg = strerror(errno);
 	    printf("441 Unable to open spool file %s: %s\r\n", outname, errmsg);
@@ -1289,7 +1300,7 @@ dopost(void)
 	ferrstr = "Missing From: header";
 
     if (havefrom && havesubject && havenewsgroups && !ferr) {
-	if (!err && 0 == chmod(outname, 0755)) {
+	if (!err && 0 == chmod(outname, 0644)) {
 	    printf("240 Article posted, now be patient\r\n");
 	    if (debugmode)
 		syslog(LOG_DEBUG, ">240 Article posted, now be patient");
@@ -1746,9 +1757,9 @@ main(int argc, char **argv)
 
     /* this reads the host name from the config file */
     if (!readconfig(1)) {
-	printf("503 Unable to read configuration file, exiting\r\n");
-	syslog(LOG_NOTICE, "Reading configuration failed, exiting "
-	       "(see syslog for more information).\n");
+	const char *m = "503 Unable to read configuration file, exiting; the server's syslog should have more information.";
+	printf("%s\r\n", m);
+	syslog(LOG_ERR, "%s", m);
 	exit(1);
     }
     freeservers();
@@ -1826,7 +1837,7 @@ main(int argc, char **argv)
     syslog(LOG_INFO, "connect from %s (%s) to %s (%s) (my fqdn: %s)",
 	    peername, peerip, fqdn, ownip, origfqdn);
 
-    if (checkpeerlocal(0) != 1 && allowstrangers == 0) {
+    if (allowstrangers == 0 && checkpeerlocal(0) != 1) {
 	unsigned int i = 5;
 	syslog(LOG_NOTICE, "Denying access from address outside the local networks. (Check config.example.)");
 	while (i) i = sleep(i);
@@ -1845,5 +1856,6 @@ main(int argc, char **argv)
 
     parser();
     (void)fflush(stdout);
+    sleep(1); /* protect against process ID induced file name collisions */
     exit(0);
 }
